/*
    Copyright (c) 2015 digitalSTROM AG, Zurich, Switzerland

    Author: Sergey 'Jin' Bostandzhyan <jin@dev.digitalstrom.org>

    This file is part of vdSM, imported from libdSvDC.

    vdSM is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 2 of the License.
    

    vdSM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __VDSM_PROPUTILS_H__
#define __VDSM_PROPUTILS_H__

#include <stdint.h>

#include "messages.pb-c.h"
#include "types.h"

//**----------------------------------------------------------------------------
// Request set property
int proputil_set_property_new(const char *dsuid,
                           Vdcapi__VdsmRequestSetProperty **property);

void proputil_set_property_free(Vdcapi__VdsmRequestSetProperty *property);


int proputil_set_property_add(Vdcapi__VdsmRequestSetProperty *property,
                          Vdcapi__PropertyElement *element);


int proputil_set_property_add_uint(Vdcapi__VdsmRequestSetProperty *property,
                               const char *key, uint64_t value);

int proputil_set_property_add_bool(Vdcapi__VdsmRequestSetProperty *property,
                               const char *key,
                               bool value);

int proputil_set_property_add_string(Vdcapi__VdsmRequestSetProperty *property,
                                 const char *key,
                                 const char *value,
                                 Vdcapi__PropertyElement** propElem);

//**----------------------------------------------------------------------------
// Request get property
int proputil_get_property_new(const char* dsuid,
                              Vdcapi__VdsmRequestGetProperty **property);

void proputil_get_property_free(Vdcapi__VdsmRequestGetProperty *property);

int proputil_get_property_add(Vdcapi__VdsmRequestGetProperty *property,
                              Vdcapi__PropertyElement *element);

//int proputil_get_property_add_uint(Vdcapi__VdsmRequestSetProperty *property,
//                            size_t index, const char *key, uint64_t value);

//**----------------------------------------------------------------------------
// Response get property
int proputil_get_property_find_element_by_name(Vdcapi__VdcResponseGetProperty *reqGetProperty,
                                                char *name,
                                                Vdcapi__PropertyElement **out);

//**----------------------------------------------------------------------------
// get Push Property
int proputil_send_push_property_find_element_by_name(Vdcapi__VdcSendPushProperty * pushProperty,
                                                     char *name,
                                                     Vdcapi__PropertyElement **out);

//**----------------------------------------------------------------------------
// Property element
int proputil_create_property_empty(Vdcapi__PropertyElement **element,
                                   const char* key);

int proputil_create_property_empty_index(Vdcapi__PropertyElement **element,
                                         int index);

int proputil_create_property_int(Vdcapi__PropertyElement **element,
                                 const char *key,
                                 int64_t value);

int proputil_create_property_uint(Vdcapi__PropertyElement **element,
                                  const char *key,
                                  uint64_t value);

int proputil_create_property_bool(Vdcapi__PropertyElement **element,
                                  const char *key,
                                  bool value);

int proputil_create_property_double(Vdcapi__PropertyElement **element,
                                    const char *key,
                                    double value);

int proputil_create_property_string(Vdcapi__PropertyElement **element,
                                    const char *key,
                                    const char *value);


int proputil_property_array_find_element_by_name(Vdcapi__PropertyElement **properties,
                                   int numOfElements,
                                   char *name,
                                   Vdcapi__PropertyElement **out);

int proputil_find_property_element_by_name(Vdcapi__PropertyElement *property,
                                          char *name,
                                          Vdcapi__PropertyElement **out);

int proputil_find_property_element_by_index(Vdcapi__PropertyElement *property,
                                          int index,
                                          Vdcapi__PropertyElement **out);

int proputil_find_property_element_by_name_index(Vdcapi__PropertyElement *property,
                                          char *name,
                                          int index,
                                          Vdcapi__PropertyElement **out);

int proputil_add_property_element(Vdcapi__PropertyElement *pParent,
                                  Vdcapi__PropertyElement *pElement);

void proputil_property_recursive_free(Vdcapi__PropertyElement *pElement);


//**----------------------------------------------------------------------------
int get_property_element_name_bool(Vdcapi__PropertyElement *property,
                                   char *name,
                                   bool *out);

int get_property_element_name_int(Vdcapi__PropertyElement *property,
                                   char *name,
                                   int64_t *out);

int get_property_element_name_uint(Vdcapi__PropertyElement *property,
                                   char *name,
                                   uint64_t *out);

int get_property_element_name_double(Vdcapi__PropertyElement *property,
                                   char *name,
                                   double *out);

int get_property_element_name_string(Vdcapi__PropertyElement *property,
                                   char *name,
                                   const char **out);


#endif//__VDSM_PROPUTILS_H__
